
package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;

public class DrawCanvas extends Canvas
    implements MouseMotionListener, MouseListener
{
    boolean             bMousePressed = false;
    DrawingObject       rectangles[];
    int                 iCurrentIndex = -1;

    boolean             bRectangleSelected = false;

    boolean             bSelectToolOn = false;
    boolean             bRectangleToolOn = false;
    public Graphics     offScreenGraphics;
    Image               offScreenImage;

    public DrawCanvas ()
    {
        addMouseListener(this);
        addMouseMotionListener(this);
        setForeground(Color.black);
    }

    public void createOffScreenGraphics()
    {
        Dimension d = getSize();
        offScreenImage = createImage(d.width, d.height);
        offScreenGraphics = offScreenImage.getGraphics();
    }

    public Graphics getOffscreenGraphics()
    {
        return offScreenGraphics;
    }

    public void setSelectToolOn ()
    {
        bSelectToolOn = true;
        bRectangleToolOn = false;
    }

    public void setRectangleToolOn ()
    {
        cleanupSelectTool();
        bSelectToolOn = false;
        bRectangleToolOn = true;
    }

    public void setDrawColor (Color color)
    {
        setForeground(color);
        offScreenGraphics.setColor(color);
    }

    public void cleanupSelectTool()
    {
        if (bRectangleSelected)
            rectangles[iCurrentIndex].deselect(this);

        iCurrentIndex = -1;
        bRectangleSelected = false;
    }


    public void mouseDragged (MouseEvent evt)
    {
        if (bMousePressed)
        {
            rectangles[iCurrentIndex].resize(this, evt.getPoint());
        }

        if (bSelectToolOn && bRectangleSelected)
        {
            if (getCursor().getType() == Cursor.DEFAULT_CURSOR)
                // move mode
                rectangles[iCurrentIndex].move(this, evt.getPoint());
            else
                // resize mode
                rectangles[iCurrentIndex].resize(this,
                                        evt.getPoint(), getCursor());
        }
    }

    public void mouseMoved (MouseEvent evt)
    {
        if (bRectangleSelected)
        {
            // check to see if mouse is over a selection handle
            rectangles[iCurrentIndex].setHandleCursor(this, evt.getPoint());
        }
    }

    public void mousePressed (MouseEvent evt)
    {
        if (bRectangleToolOn)
        {
            bMousePressed = true;
            iCurrentIndex = addNewRectangle(evt.getPoint());
            rectangles[iCurrentIndex].setColor(getForeground());
        }

        if (bSelectToolOn && bRectangleSelected)
        {
            rectangles[iCurrentIndex].setHandleCursor(this, evt.getPoint());
            if (getCursor().getType() == Cursor.DEFAULT_CURSOR)
                // set up for move mode
                rectangles[iCurrentIndex].setMoveAnchor(evt.getPoint());
        }
    }

    public void mouseReleased (MouseEvent evt)
    {
        bMousePressed = false;
    }

    public void mouseEntered (MouseEvent evt)
    {

    }

    public void mouseExited (MouseEvent evt)
    {

    }

    public void mouseClicked (MouseEvent evt)
    {
        if (bSelectToolOn)
        {
            if (rectangles != null)
            {
                // deselect the rectangle that is selected
                if (bRectangleSelected)
                    rectangles[iCurrentIndex].deselect(this);

                iCurrentIndex = -1;
                bRectangleSelected = false;

                Point p = evt.getPoint();
                for (int iCount = 0; iCount < rectangles.length; iCount++)
                {
                    if (rectangles[iCount].isBorderClicked(this, p))
                    {
                        rectangles[iCount].select(this);
                        iCurrentIndex = iCount;
                        bRectangleSelected = true;
                        break;
                    }
                }

                if (bRectangleSelected)
                {
                    // check to see if right mouse button clicked
                    if (evt.getModifiers() == InputEvent.BUTTON3_MASK)
                    {
                        ColorPopupMenu colorPopupMenu =
                         new ColorPopupMenu(rectangles[iCurrentIndex], this,
                                            rectangles[iCurrentIndex].getColor());
                        add(colorPopupMenu);
                        colorPopupMenu.show(this, evt.getX(), evt.getY());
                    }
                }
            }
        }
    }

    int addNewRectangle(Point p)
    {
        if (rectangles == null)
        {
            rectangles = new RectangleObject[1];
            rectangles[0] = new RectangleObject(p);
        }
        else
        {
            DrawingObject newRectangles[] =
                new DrawingObject[rectangles.length + 1];

            for (int iCount = 0; iCount < rectangles.length; iCount++)
            {
                newRectangles[iCount] = rectangles[iCount];
            }

            newRectangles[newRectangles.length - 1] =
                                    new RectangleObject(p);
            rectangles = newRectangles;
        }

        return (rectangles.length - 1);

    }

    public void paint (Graphics g)
    {
        if (offScreenImage != null)
        {
            g.drawImage(offScreenImage, 0,0,this);
        }
    }

    public void update (Graphics g)
    {
        if (rectangles != null)
        {
            for (int iCount = 0; iCount < rectangles.length; iCount++)
            {
                rectangles[iCount].draw(this, offScreenGraphics);
            }
        }
        paint(g);
    }

    public void print (PrintJob printJob)
    {
        if (printJob != null)
        {
            Graphics pg = printJob.getGraphics();
            if (pg != null)
            {
                paint(pg);
                pg.dispose();
            }

            printJob.end();
        }
    }

    public void delete ()
    {
        DrawingObject rectangleToDelete = rectangles[iCurrentIndex];

        if (bSelectToolOn && bRectangleSelected)
        {
            // remove from rectangles array
            if (rectangles.length == 1)
            {
                rectangles = null;
            }
            else
            {
                int iOffset = 0;
                DrawingObject newRectangles[] =
                    new DrawingObject[rectangles.length - 1];

                for (int iCount = 0; iCount < rectangles.length; iCount++)
                {
                    if (iCount == iCurrentIndex)
                        iOffset++;
                    else
                        newRectangles[iCount - iOffset] =
                                            rectangles[iCount];
                }
                rectangles = newRectangles;
            }

            rectangleToDelete.deselect(this);
            rectangleToDelete.erase(this);

            iCurrentIndex = -1;
            bRectangleSelected = false;
        }
    }
}