
package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;

/**
 * This class is a frame that will demonstrate the following features:
 *
 * - Mouseless Operation
 * - Popup Menu
 * - Printing
 * - ScrollPane
 * - Cursor
 *
 * @author Eric Monk
 * @version 1.0
 *
 */

public class DrawRectangleFrame extends Frame
        implements ActionListener, ItemListener
{
    DrawCanvas  drawCanvas = new DrawCanvas();

    MenuBar     menuBar = new MenuBar();
    Menu        mnuFile = new Menu("File");
    Menu        mnuEdit = new Menu("Edit");
    Menu        mnuColor = new Menu("Color");
    Menu        mnuTool = new Menu("Tool");

    MenuItem mnuPrint = new MenuItem("Print", new MenuShortcut('p'));
    MenuItem mnuExit = new MenuItem("Exit");

    MenuItem mnuDelete = new MenuItem("Delete", new MenuShortcut('d'));

    CheckboxMenuItem mnuColorBlue = new CheckboxMenuItem("Blue");
    CheckboxMenuItem mnuColorGreen = new CheckboxMenuItem("Green");
    CheckboxMenuItem mnuColorRed = new CheckboxMenuItem("Red");
    CheckboxMenuItem mnuColorBlack = new CheckboxMenuItem("Black");

    CheckboxMenuItem mnuPointerTool = new CheckboxMenuItem("Pointer");
    CheckboxMenuItem mnuRectangleTool = new CheckboxMenuItem("Rectangle");

    public DrawRectangleFrame (String sTitle)
    {
        super (sTitle);


        ScrollPane scrollPane =
            new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);

        Adjustable vadjust = scrollPane.getVAdjustable();
        Adjustable hadjust = scrollPane.getHAdjustable();
        hadjust.setUnitIncrement(10);
        vadjust.setUnitIncrement(10);

        drawCanvas.setSize(400,400);
        scrollPane.add(drawCanvas);

        setLayout(new BorderLayout());
        add("Center", scrollPane);

        mnuFile.add(mnuPrint);
        mnuFile.addSeparator();
        mnuFile.add(mnuExit);

        mnuPrint.addActionListener(this);
        mnuExit.addActionListener(this);

        mnuEdit.add(mnuDelete);

        mnuDelete.addActionListener(this);

        mnuColor.add(mnuColorBlue);
        mnuColor.add(mnuColorGreen);
        mnuColor.add(mnuColorRed);
        mnuColor.add(mnuColorBlack);

        mnuColorBlue.addItemListener(this);
        mnuColorGreen.addItemListener(this);
        mnuColorRed.addItemListener(this);
        mnuColorBlack.addItemListener(this);
        mnuColorBlack.setState(true);

        mnuTool.add(mnuPointerTool);
        mnuTool.add(mnuRectangleTool);

        mnuPointerTool.addItemListener(this);
        mnuRectangleTool.addItemListener(this);
        mnuPointerTool.setState(true);

        menuBar.add(mnuFile);
        menuBar.add(mnuEdit);
        menuBar.add(mnuColor);
        menuBar.add(mnuTool);

        setMenuBar(menuBar);

        setSize(300,300);
        show();
        drawCanvas.createOffScreenGraphics();
    }

    public void actionPerformed (ActionEvent evt)
    {
        Object target = evt.getSource();
        if (target == mnuExit)
        {
            dispose();
            System.exit(0);
        }

        else if (target == mnuPrint)
        {
            PrintJob printJob = getToolkit().getPrintJob(this,
                                     "Print Rectangles", null);
            drawCanvas.print(printJob);
        }

        else if (target == mnuDelete)
        {
            drawCanvas.delete();
        }
    }

    public void itemStateChanged (ItemEvent evt)
    {
        Object target = evt.getSource();
        if (evt.getStateChange() == ItemEvent.SELECTED)
        {
            if (target == mnuPointerTool)
            {
                mnuRectangleTool.setState(false);
                drawCanvas.setSelectToolOn();
            }
            else if (target == mnuRectangleTool)
            {
                mnuPointerTool.setState(false);
                drawCanvas.setRectangleToolOn();
            }
            else if (target == mnuColorBlue)
            {
                drawCanvas.setDrawColor(Color.blue);
                deselectColorItems(target);
            }
            else if (target == mnuColorRed)
            {
                drawCanvas.setDrawColor(Color.red);
                deselectColorItems(target);
            }
            else if (target == mnuColorGreen)
            {
                drawCanvas.setDrawColor(Color.green);
                deselectColorItems(target);
            }
            else if (target == mnuColorBlack)
            {
                drawCanvas.setDrawColor(Color.black);
                deselectColorItems(target);
            }
        }
        else
        {
            // never allow all tools to be deselected
            ((CheckboxMenuItem)target).setState(true);
        }
    }

    void deselectColorItems (Object oKeepSelected)
    {
        if (mnuColorGreen != oKeepSelected)
            mnuColorGreen.setState(false);

        if (mnuColorBlue != oKeepSelected)
            mnuColorBlue.setState(false);

        if (mnuColorRed != oKeepSelected)
            mnuColorRed.setState(false);

        if (mnuColorBlack != oKeepSelected)
            mnuColorBlack.setState(false);
    }

    public static void main (String args[])
    {
        new DrawRectangleFrame("Draw Rectangle Program");
    }
}