package jwiley.chp10;

import java.awt.*;
import java.util.Vector;
import java.util.StringTokenizer;

public class Grid implements LayoutManager
{
    Vector vCoordinates = new Vector();
    Vector vComponents = new Vector();
    int iRows;
    int iCols;
    int iXInsets;
    int iYInsets;

    public Grid(int iCols, int iRows)
    {
        this(iCols, iRows, 0, 0);
    }

    public Grid(int iCols, int iRows, int iXInsets, int iYInsets)
    {
        if (iRows < 1)
            iRows = 1;
        else
            this.iRows = iRows;

        if (iCols < 1)
            iCols = 1;
        else
            this.iCols = iCols;

        if (iXInsets < 0)
            iXInsets = 0;
        else
            this.iXInsets = iXInsets;

        if (iYInsets < 0)
            iYInsets = 0;
        else
            this.iYInsets = iYInsets;
    }


    public void addLayoutComponent(String sCoordinates, Component comp)
    {
        vCoordinates.addElement(sCoordinates);
        vComponents.addElement(comp);
    }

    public void removeLayoutComponent(Component comp)
    {
        int iCount = vComponents.indexOf(comp);
        if (iCount != -1)
        {
            vCoordinates.removeElementAt(iCount);
            vComponents.removeElementAt(iCount);
        }
    }

    public Dimension preferredLayoutSize(Container target)
    {
        int     iMaxHeight = 0;
        int     iMaxWidth = 0;

        // find out largest preferred size for width and height
        int iNumComponents = vComponents.size();
        for (int iCount = 0; iCount < iNumComponents; iCount++)
        {
            Component comp = (Component)vComponents.elementAt(iCount);
            Dimension d = comp.getPreferredSize();

            if (d.width > iMaxWidth)
                iMaxWidth = d.width;

            if (d.height > iMaxHeight)
                iMaxHeight = d.height;
        }

        // add grid getInsets
        iMaxHeight += iYInsets * 2;
        iMaxWidth += iXInsets * 2;

        // multiply by cols and rows
        iMaxHeight *= iRows;
        iMaxWidth *= iCols;

        // add target getInsets
        Insets insets = target.getInsets();

        iMaxHeight += insets.bottom + insets.top;
        iMaxWidth += insets.left + insets.right;

        return (new Dimension(iMaxWidth, iMaxHeight));
    }

    public Dimension minimumLayoutSize(Container target)
    {
        int     iMaxHeight = 0;
        int     iMaxWidth = 0;

        // find out largest preferred size for width and height
        int iNumComponents = vComponents.size();
        for (int iCount = 0; iCount < iNumComponents; iCount++)
        {
            Component comp = (Component)vComponents.elementAt(iCount);
            Dimension d = comp.getMinimumSize();

            if (d.width > iMaxWidth)
                iMaxWidth = d.width;

            if (d.height > iMaxHeight)
                iMaxHeight = d.height;
        }

        // add grid getInsets
        iMaxHeight += iYInsets * 2;
        iMaxWidth += iXInsets * 2;

        // multiply by cols and rows
        iMaxHeight *= iRows;
        iMaxWidth *= iCols;

        // add target getInsets
        Insets insets = target.getInsets();

        iMaxHeight += insets.bottom + insets.top;
        iMaxWidth += insets.left + insets.right;

        return (new Dimension(iMaxWidth, iMaxHeight));
    }

    public void layoutContainer(Container target)
    {
        Insets insets = target.getInsets();
        int iNumComponents = vComponents.size();
        Dimension d = target.getSize();
        d.width -= insets.left + insets.right;
        d.height -= insets.top + insets.bottom;

        float fCellWidth = d.width / iCols;
        float fCellHeight = d.height / iRows;

        for (int iCount = 0; iCount < iNumComponents; iCount++)
        {
            Component comp = (Component)vComponents.elementAt(iCount);
            Dimension pf = comp.getPreferredSize(); //
            StringTokenizer st =
                new StringTokenizer((String)vCoordinates.elementAt(iCount),
                                                                    ",;\t");

            int iLeft = Integer.valueOf(st.nextToken()).intValue();
            int iTop = Integer.valueOf(st.nextToken()).intValue();

            int iWidth, iHeight;

            boolean bUsePreferredSize = false;

            if (st.hasMoreElements())
            {
                String sNextElement = st.nextToken();
                if (sNextElement.equals("p"))
                {
                    bUsePreferredSize = true;
                    iWidth = 1;
                    iHeight = 1;
                }
                else
                {
                    iWidth = Integer.valueOf(sNextElement).intValue();
                    iHeight = Integer.valueOf(st.nextToken()).intValue();
                    if (st.hasMoreElements())
                    {
                        sNextElement = st.nextToken();
                        if (sNextElement.equals("p"))
                            bUsePreferredSize = true;
                    }
                }
            }
            else
            {
                iWidth = 1;
                iHeight = 1;
            }

            int x1 = 0, y1 = 0, x2 = 0, y2 = 0;

            if (bUsePreferredSize)
            {
                int iWholeWidth = (int)(fCellWidth * iWidth)
                                                   - (2 * iXInsets);
                int iWholeHeight = (int)(fCellHeight * iHeight)
                                                   - (2 * iYInsets);
                x1 = (int)(fCellWidth * (iLeft - 1)) + insets.left
                          + iXInsets + ((iWholeWidth - pf.width) / 2);
                y1 = (int)(fCellHeight * (iTop - 1)) + insets.top
                          + iYInsets + ((iWholeHeight - pf.height) / 2);
                x2 = pf.width;
                y2 = pf.height;
            }
            else
            {
                x1 = (int)(fCellWidth * (iLeft - 1)) + insets.left
                                                          + iXInsets;
                y1 = (int)(fCellHeight * (iTop - 1)) + insets.top
                                                          + iYInsets;
                x2 = (int)(fCellWidth * iWidth) - (2 * iXInsets);
                y2 = (int)(fCellHeight * iHeight) - (2 * iYInsets);
            }

            comp.setBounds(x1, y1, x2, y2);

        }
    }
}


