/** loadImage.java */
package jwiley.chp10;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * Class that represents a window with a single image displayed in it.
 * @version 1.1
 * @author Michael C. Daconta
 */
class imageFrame extends Frame
{
    /** The image to display. */
    Image theImg;

    /** Adapter class to shutdown application when close
        box clicked on window. */
    class ShutdownAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent we)
        { System.exit(0); }
    }

    /**
     * Constructor to create the frame.
     * @param anImg the Image to display.
     * @param sizeToImage flag to determine size of window frame. If true
     *        the frame will be sized to the image width and height.
     */
    imageFrame(Image anImg, boolean sizeToImage)
    {
        super("Image Frame");
        theImg = anImg;
        if (sizeToImage)
            setSize(anImg.getWidth(this) + 1,anImg.getHeight(this) + 1);
        else
            setSize(200,200);
        addWindowListener(new ShutdownAdapter());
        show();
    }

    /** Method to paint image in window. */
    public void paint(Graphics g)
    {
        Dimension d = getSize();
        g.drawImage(theImg,0,0,d.width, d.height, this);
    }
}

/**
 * Class that wraps the main method and responds to command line args.
 */
public class LoadImage
{
    /** Main method of standalone program. */
    public static void main(String args[])
    {
        boolean fitWindowToImage=false;
        boolean waitForImageToLoad=false;

        if (args.length < 1)
        {
            System.out.println("USAGE: java loadImage " +
                               "<imageFileName> [-fit] [-wait]");
            System.exit(1);
        }

        // set flags if necessary
        for (int i = 0; i < args.length; i++)
        {
            if (args[i].equals("-fit"))
                fitWindowToImage = true;
            else if (args[i].equals("-wait"))
                waitForImageToLoad = true;
        }

        // check if file exists
        File imgFile = new File(args[0]);
        if (imgFile.exists())
        {
            // load the image
            Image theImage = (Toolkit.getDefaultToolkit()).getImage(args[0]);

            if (waitForImageToLoad || fitWindowToImage)
            {
                // create a MediaTracker
                MediaTracker theTracker = new MediaTracker(new Frame());
                theTracker.addImage(theImage,0);
                try
                {
                    theTracker.waitForAll();
                } catch(Exception e)
                  {
                    System.out.println("Error loading image.");
                    System.exit(1);
                  }

                if (theTracker.isErrorAny())
                {
                    System.out.println("Error loading image.");
                    System.exit(1);
                }
            }

            // display the image
            new imageFrame(theImage, fitWindowToImage);
        }
        else
        {
            System.out.println("Image File <" + args[0] + "> does not exist.");
        }
    }
}

