
package jbook.chp12;

import java.net.Socket;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class SearchClient
{
    String  sHostName = "";
    int     iPort = 0;

    public SearchClient(String sHostName, int iPort)
    {
        this.sHostName = sHostName;
        this.iPort = iPort;
    }

    public Document[] performSearch (String sSearchText,
                                   boolean bTitleSearch,
                                   boolean bKeywordSearch,
                                   boolean bTextSearch)
    {
        Socket      sock = null;
        Document    docs[] = null;

        try
        {
            sock = new Socket(sHostName, iPort);

            DataInputStream disStream =
                    new DataInputStream(sock.getInputStream());

            DataOutputStream dosStream =
                    new DataOutputStream(sock.getOutputStream());

            dosStream.writeUTF("<SEARCH>");
            dosStream.writeUTF(sSearchText);
            dosStream.writeBoolean(bTitleSearch);
            dosStream.writeBoolean(bKeywordSearch);
            dosStream.writeBoolean(bTextSearch);

            int iNumDocs = disStream.readInt();
            if (iNumDocs > 0)
            {
                docs = new Document[iNumDocs];
                for (int iCount = 0; iCount < iNumDocs; iCount++)
                {
                    String sFileName = disStream.readUTF();
                    String sTitle = disStream.readUTF();

                    docs[iCount] = new Document(sFileName,
                                                sTitle);
                }
            }

            sock.close();

        } catch (IOException ioe)
          {
            System.out.println("Error in performing search." +
                               "\nError: " + ioe.toString());
          }

        return (docs);
    }

    public String getDocumentText (String sFileName)
    {
        Socket      sock = null;
        String      sDocText = null;

        try
        {
            sock = new Socket(sHostName, iPort);

            DataInputStream disStream =
                    new DataInputStream(sock.getInputStream());

            DataOutputStream dosStream =
                    new DataOutputStream(sock.getOutputStream());

            dosStream.writeUTF("<GET-DOC>");
            dosStream.writeUTF(sFileName);
            sDocText = disStream.readUTF();

            sock.close();

        } catch (IOException ioe)
          {
            System.out.println("Error in getting document." +
                               "\nError: " + ioe.toString());
          }

        return (sDocText);
    }

}