
import java.security.*;
import java.security.acl.*;		// The Acl and AclEntry interfaces
import sun.security.acl.*;		// The sun implementation.
import java.util.Enumeration;

/**
 * public class testAcl
 * @author A. Saganich
 * @version 1.0
 *
 * the testAcl class exercises the Priciple, permission, group, 
 * Acl and AclEntry classes.     
 *
 *  @see java.security.acl.Acl
 *  @see java.security.acl.AclEntry
 *  @see java.security.acl.Principal
 *  @see java.security.acl.Group
 *  @see java.security.acl.Permission
 */
public class TestAcl
{

	// the employees
	Principal ceo 			= null;
	Principal cfo 			= null;
	
	Principal researcher1	= null;
	Principal researcher2	= null;
	
	Principal accountant1	= null;
	Principal accountant2	= null;
	Principal sneakyEmployee= null;
	
	Principal employees[]   = new Principal[7];
	
	
	Permission read			= new PermissionImpl("Read");
	Permission write		= new PermissionImpl("Write");
	Permission create		= new PermissionImpl("Create");
	Permission delete		= new PermissionImpl("Delete");
	
	// The groups they belong too.
	Group	  	financialGroup	= null;
	Group	  	researchGroup	= null;
	
	
	// The resources the users may access
	Acl			printers		= null;
	Acl			financialRcrds	= null;
	Acl			researchInfo	= null;
	TestAcl()
	{
		ceo = new PrincipalImpl("CEO"); employees[0] = ceo;
		cfo = new PrincipalImpl("CFO"); employees[1] = cfo;
		
		researchGroup  = new GroupImpl("Pure R&D");
		researcher1	= new PrincipalImpl("Software Developer"); employees[2] = researcher1;
		researcher2	= new PrincipalImpl("Software Manager");   employees[3] = researcher2;
		researchGroup.addMember(researcher1);
		researchGroup.addMember(researcher2);
		researchGroup.addMember(ceo);
		researchGroup.addMember(cfo);
	
		accountant1	= new PrincipalImpl("Accounting"); employees[4] = accountant1;
		accountant2	= new PrincipalImpl("Bookeeping"); employees[5] = accountant2;
		sneakyEmployee= new PrincipalImpl("Mr Spy");   employees[6] = sneakyEmployee;
		financialGroup = new GroupImpl("Finance");
		financialGroup.addMember(accountant1);
		financialGroup.addMember(accountant2);
		financialGroup.addMember(sneakyEmployee);
		financialGroup.addMember(ceo);
		financialGroup.addMember(cfo);
		
		// Now create the acls which match resources.
		printers 		= new AclImpl(ceo,"Printers");
		financialRcrds  = new AclImpl(cfo,"Finacial Records");
		researchInfo	= new AclImpl(ceo,"Research Information");		
	
		// And add entries for each		

		try
		{
			AclEntry entry = new AclEntryImpl(researchGroup);
			entry.addPermission(read);
			entry.addPermission(write);
			entry.addPermission(create);
			entry.addPermission(delete);
			printers.addEntry(ceo,entry);
			researchInfo.addEntry(ceo,entry);
			entry = new AclEntryImpl(financialGroup);
			entry.addPermission(read);
			researchInfo.addEntry(ceo,entry);
			
		}
		catch (NotOwnerException e)
		{
			System.out.println("Unexpected error adding research permissions" + e);
		}
		try
		{
			AclEntry entry = new AclEntryImpl(financialGroup);
			entry.addPermission(read);
			entry.addPermission(write);
			entry.addPermission(create);
			entry.addPermission(delete);
			
			printers.addEntry(ceo,entry);
			financialRcrds.addEntry(cfo,entry);
			
			// Except sneaky he's denied!
			entry = new AclEntryImpl(sneakyEmployee);
			entry.addPermission(read);
			entry.addPermission(write);
			entry.addPermission(create);
			entry.addPermission(delete);
			entry.setNegativePermissions();
			financialRcrds.addEntry(cfo,entry);
			
			// Grant read access to the research group.
			entry = new AclEntryImpl(financialGroup);
			entry.addPermission(read);
			financialRcrds.addEntry(cfo,entry);

			// Except sneaky!			
			entry = new AclEntryImpl(sneakyEmployee);
			entry.addPermission(read);
			entry.setNegativePermissions();
			researchInfo.addEntry(ceo,entry);
			
			
		}
		catch (NotOwnerException e)
		{
			System.out.println("Unexpected error adding financial permissions" + e);
		}
	
	}
	
	// list the permissions in the acl's
	public void listPermissions()
	{
		if ( printers != null )
		{
			for (int i = 0; i < 7; i++)
			{
				System.out.println("Printer permissions  for " + employees[i]);
				Enumeration e = printers.getPermissions(employees[i]);
				while (e.hasMoreElements())
				{
					System.out.println("'" + e.nextElement() + "'");
				}
			}
		}
		if ( financialRcrds != null )
		{
			for (int i = 0; i < 7; i++)
			{
				System.out.println("financialRcrds permissions  for " + employees[i]);
				Enumeration e = financialRcrds.getPermissions(employees[i]);
				while (e.hasMoreElements())
				{
					System.out.println("'" + e.nextElement() + "'");
				}
			}
		}
		if ( researchInfo != null )
		{
			for (int i = 0; i < 7; i++)
			{
				System.out.println("researchInfo permissions  for " + employees[i]);
				Enumeration e = researchInfo.getPermissions(employees[i]);
				while (e.hasMoreElements())
				{
					System.out.println("'" + e.nextElement() + "'");
				}
			}
		}
	}
	
	
	// List the members of the two groups.
	public void listGroups()
	{
		if ( financialGroup != null)
		{
			System.out.println("Members of the financial group");
			Enumeration e =financialGroup.members();
			while (e.hasMoreElements())
			{
				System.out.println(e.nextElement());
			}
		}
		if ( researchGroup != null)
		{
			System.out.println("Members of the research group");
			Enumeration e =researchGroup.members();
			while (e.hasMoreElements())
			{
				System.out.println(e.nextElement());
			}
		}
		
		
	}

	static public void main(String args[])
	{
		TestAcl testAcl = new TestAcl();
		//testAcl.listGroups();
		testAcl.listPermissions();
	}


}