
package jwiley.chp18;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.sql.SQLException;

/**
 * Class to demonstrate the use of basic jdbc functionality
 *
 * @author  Eric Monk
 */

public class SimpleMetaData
{
    public static void main (String args[])
    {
        try
        {
            // Load database driver
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch (ClassNotFoundException cnfe)
          {
            System.out.println(cnfe.toString());
            System.exit(0);
          }

        try
        {
            // Establish connection
            String sUrl = "jdbc:odbc:ExampleDb";
            Connection con = DriverManager.getConnection(sUrl);

            // Create and send statement
            String sQuery = "SELECT * FROM BOOKS";

            Statement stmt = con.createStatement();
            ResultSet rslt = stmt.executeQuery(sQuery);

            ResultSetMetaData rsmd = rslt.getMetaData();
            int iColumnCount = rsmd.getColumnCount();

            System.out.println("Field\tType");
            System.out.println("-----\t----");
            for (int iCount = 1; iCount <= iColumnCount; iCount++)
            {
                int iWidth = rsmd.getColumnDisplaySize(iCount);
                String sName = rsmd.getColumnName(iCount);
                int iType = rsmd.getColumnType(iCount);

                String sType = null;
                switch (iType)
                {
                    case Types.TINYINT:
                    case Types.SMALLINT:
                    case Types.INTEGER:
                    case Types.BIGINT:
                        sType = "Integer";
                        break;

                    case Types.CHAR:
                    case Types.VARCHAR:
                        sType = "Char";
                        break;
                }

                String sDescription = sType;
                if (sType.equals("Char"))
                    sDescription = sDescription + "(" + iWidth + ")";

                System.out.println(sName + "\t" + sDescription);
            }

            // Cleanup
            rslt.close();
            stmt.close();

        } catch (SQLException sqle)
          {
            while (sqle != null)
            {
                System.out.println ("SQLState: " +
                                    sqle.getSQLState ());
                System.out.println ("Message:  " +
                                    sqle.getMessage ());
                System.out.println ("ErrorCode:   " +
                                    sqle.getErrorCode ());
                sqle = sqle.getNextException ();
                System.out.println ("");
            }
          }

    }
}
