// PointerEquiv.java
package jwiley.chp2;

class simpleObject {
    int a;
    public simpleObject left, right;

    simpleObject(int inA)
    {
        a=inA;
        left = right = null;
    }

    int getA()
    {
        return a;
    }

    void setA(int inA)
    {
        a = inA;
    }

    public String toString()
    {
        return new String("a: " + a);
    }
}

class PointerEquiv {
    static void swap(Object a, Object b)
    {
            Object tmp = a;
            a = b;
            b = tmp;
    }

    public static void main(String args[])
    {
        /* 1) Like pointers, all objects and arrays
              are dynamically allocated. */
        int arr[] = new int[10];
        simpleObject myObj = new simpleObject(5);

        /* 2) An uninitialized array or object is implemented
              via a handle (pointer pointer) that can point
              to another object. */
        int arr2[] = arr;
        simpleObject yourObj = myObj;
        System.out.println("yourObj is " + yourObj.toString());

        /* 3) null is a reserved keyword identical to
              NULL in usage. */
        int arr3[] = null;
        arr = null; /* causes arr to point to null */

        /* 4) Arrays and Objects are placed on the stack as
              a handle (not pass by value).  However, there is
              no Pass by Reference in Java at this time. */
        simpleObject otherObject = new simpleObject(20);
        System.out.println("Before swap, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());
        swap(myObj, otherObject);
        System.out.println("After swap, myObj is " + myObj.toString() +
                           " otherObject is " + otherObject.toString());

        /* 5) There is no explicit free or delete.  Java
              memory is garbage collected. */
        System.gc();  /* force garbage collection to occur NOW. */

        /* 6) Since all Java objects are descendants of a
              single Object class.  An unitialized Object
              can act like a void pointer. */
        Object voidp = otherObject;
        ((simpleObject) voidp).setA(100);
        System.out.println("voidp points to " +
                            ((simpleObject) voidp).toString());
        System.out.println("Just as otherObject now is " +
                                otherObject.toString());

        /* 7) Self-referential structures are possible by
              using non-instantiated objects. */
        simpleObject root = new simpleObject(1000);
        root.left = new simpleObject(500);
        root.right = new simpleObject(2000);

        /* 8) There is NO equivalent to function pointers
              in Java. Safety prevailed as a design goal. */
    }
}
