package jwiley.chp21;

import java.applet.*;		// For applet
import java.awt.*;			// for layout etc
import java.net.*;			// for URL
import java.io.IOException;	// for IO Exceptions
import java.media.*;		// for the Java media class

/**
 *  Player4.java
 *  This example demonstrates the use of a single player to control other players.
 *
 *  @version 1.0
 *  @author A.J. Saganich Jr.
 */
public class Player4
	extends Applet 
	implements ControllerListener
	//  requires controllerUpdate(ControllerEvent event) be implemented
{
	private static final 
	int 		ctPlayers 	= 3;
	Player 		players[] 	= null;
	Panel		playerPanel	= null;
	Component 	vc 			= null;
	Component	controls	= null;
	boolean 	fRunning 	= false;		// Not initially running

	/**
	 * Read the name of the media file and then begin processing
	 */
	public void init()
	{
		String inputFile = null;
		URL inputURL	 = null;

		/**
		 * the inputFile parameter contains the name
		 * of the media file to play.
		 */
		inputFile = getParameter("inputFile");
		if ( null == inputFile)
		{
			System.out.println("Invalid media file parameter");
			return;
		}

		System.out.println("Running on file: "+ inputFile);
		// Create an url from the file name and the url to the
		// document containing this applet.
		try
		{
			inputURL = new URL(getDocumentBase(), inputFile);
		}
		catch (MalformedURLException e)
		{
			System.out.println("Invalid media file URL!");
		}

		// Create the players
		try
		{
			players = new Player[ctPlayers];
			// Create a player that corresponds to the input type.
			for (int i = 0; i < ctPlayers; i++)
			{
				players[i] = Manager.createPlayer(inputURL);
				if ( players[i] == null )
				{
					System.out.println("Could not create player for " + inputURL);
				}
			}


			// We only need capture events on the master.
			if(players[0] != null)
			{
				players[0].addControllerListener(this);
			}
			
			// Realize all the players
			int i;
			for (i = 0;  i < ctPlayers; i++)
			{
				players[i].realize();
			}

			// Wait for them all to realize
			while (true)
			{
				for (i = 0;  i < ctPlayers; i++)
				{
					if (players[i].getState() != Player.Realized) break;
				}
				if ( i == ctPlayers ) break;
			}
	
			// build a panel for the clips
			playerPanel = new Panel();
			playerPanel.setLayout(new GridLayout(ctPlayers,1)); // n x 1 grid( a stack)

			
			// Use a border based layout for the whole thing
			setLayout(new BorderLayout());
			add ("Center",playerPanel);
			// Add them to the panel and make player[0] control all others
			for (i = 0;  i < ctPlayers; i++)
			{
				playerPanel.add(players[i].getVisualComponent());
				if (i != 0)
					players[0].addController(players[i]);
			}
		}
		// Using the addController method may throw an IncompatibleTimeBaseException exception
		catch( IncompatibleTimeBaseException e )
		{
			System.out.println("Unexpected TimeBase exception " + e);
		}
		catch(IOException e)
		{
			System.out.println("IO exception creating aPlayer for" + inputURL);
		}
	}



	/**
	 * Start method
	 * Note that this method attempts to start the prefetch of the
	 * clip but does not start the player playing.  When the prefetch
	 * completes the a event will be generated and the controllerUpdate
	 * method called which will start the clip playing.
	 */
	public void start()
	{
		players[0].prefetch();

	}

	/**
	 * Stop method
	 * stop the player playing, if allocated, and release its resources.
	 */
	public void stop()
	{
		players[0].stop();
		players[0].deallocate();
	}

	/**
	 * controllerUpdate method
	 * required by the ControlllerLister interface
	 * called whenever a media event occurs
	 */
	public synchronized void controllerUpdate(ControllerEvent event)
	{

		System.out.println("ControllerUpdate with event " + event.toString());

		// When the aPlayer is Realized, get the visual component
		// and add it to the Applet
		if (event instanceof RealizeCompleteEvent)
		{
			// force the applet to draw the image and the controls

			controls = players[0].getControlPanelComponent();
			if ( controls != null)
			{	
				controls.setBackground(Color.lightGray);
				add("North", controls);
			}
			validate();
		}
		else if (event instanceof PrefetchCompleteEvent)
		{
			players[0].start();
		}
		else if (event instanceof EndOfMediaEvent)
		{
			players[0].setMediaTime(0);
			players[0].start();
		}
		else 
		{
			// Some other event occured
			// Adding fast forward and rewind events is left as an execise
			// to the reader.
		
		}
	} // End method controllerUpdate
} // End class Player3
