/** DatabaseTableDataModel.java */
package jwiley.chp23;

import java.sql.*;
import java.util.Vector;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;

/**
 * DataModel for the JTable component.
 * @author Eric Monk
 * @author Mike Daconta
 */
public class DatabaseTableDataModel implements TableDataModel
{
    /** column headings. */
    private String[]    sColumnHeadings;
    /** internal data object. */
    private Object[][]  data;
    /** number of rows in table. */
    private int         iNumRows;

    /** Constructor to build data for JTable by querying any jdbc
     *  table.
     * @param sDatabaseURL - url connection to database in jdbc format.
     * @param sTable - table name to query.
     */
    public DatabaseTableDataModel (String sDatabaseURL, String sTable) throws Exception
    {
        try
        {
            // Load database driver
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
        } catch (ClassNotFoundException cnfe)
          {
            throw new Exception("Error initializing database driver. " +
                                "Error: " + cnfe.toString());
          }

        // getMetaData - column headings
        // get all data from table

        try
        {
            // Establish connection
            Connection con = DriverManager.getConnection(sDatabaseURL);

            // Create and send statement
            String sQuery = "SELECT * FROM " + sTable;

            Statement stmt = con.createStatement();
            ResultSet rslt = stmt.executeQuery(sQuery);
            ResultSetMetaData rsmd = rslt.getMetaData();

            sColumnHeadings = new String[rsmd.getColumnCount()];

            for (int iCount = 0; iCount < sColumnHeadings.length; iCount++)
                sColumnHeadings[iCount] = rsmd.getColumnName(iCount + 1);

            Vector vRows = new Vector();

            // Process data
            while (rslt.next())
            {
                String sDataForRow[] = new String[sColumnHeadings.length];

                for (int iCount = 0; iCount < sColumnHeadings.length; iCount++)
                {
                    sDataForRow[iCount] = rslt.getString(iCount + 1);
                }

                vRows.addElement(sDataForRow);
                iNumRows++;
            }

            data = new String[vRows.size()][sColumnHeadings.length];

            for (int iCount = 0; iCount < vRows.size(); iCount++)
                data[iCount] = (String[])vRows.elementAt(iCount);

            // Cleanup
            rslt.close();
            stmt.close();
            con.close();

        } catch (SQLException sqle)
          {
            while (sqle != null)
            {
                System.out.println ("SQLState: " +
                                    sqle.getSQLState ());
                System.out.println ("Message:  " +
                                    sqle.getMessage ());
                System.out.println ("ErrorCode:   " +
                                    sqle.getErrorCode ());
                sqle = sqle.getNextException ();
            }
            throw new Exception ("Database error has occurred. " +
                                 "Error: " + sqle.toString());

          }
    }

    /** accessor to get the column headings. */
    public String[] getColumnHeadings()
    {
        return (sColumnHeadings);
    }

    /** required method to implement TableDataModel.
        returns the number of rows available. */
    public int getRowCount()
    {
        return (iNumRows);
    }

    /** required method to implement TableDataModel.
     *  Retrieves a value at this column and row.
     *  @param columnIdentifier - object that indentifiel the column.
     *  @param iRowIndex - row to access.
     */
    public Object getValueAt(Object columnIdentifier, int iRowIndex)
    {
        if (data != null && sColumnHeadings != null)
        {
            for (int iCount = 0; iCount < sColumnHeadings.length; iCount++)
            {
                if (sColumnHeadings[iCount].equals(columnIdentifier))
                    return data[iRowIndex][iCount];
            }
        }
        return ("null");
    }
}