// ThisDemo.java
package jwiley.chp3;

/**
 * Class to represent an automobile. Demonstrates
 * the use of the this reference.
 */
class Automobile
{
    /** model of car. */
    String sModel;
    /** Capacity of fuel tank in gallons. */
    int iTankCapacity;
    /** miles car has travelled. */
    int iOdometer;
    /** fuel efficiency of car. */
    int iMilesPerGallon;
    /** gas tank. */
    FuelTank gasStorage;

    /** no-arg constructor. NOTICE use of this. */
    Automobile()
    {
        this("Unknown",10, 20);
    }

    /** overloaded constructor. */
    Automobile(String sModelName)
    {
        this(sModelName,10, 20);
    }

    /** overloaded constructor. */
    Automobile(String sModelName, int iNumGallons)
    {
        this(sModelName,iNumGallons, 20);
    }

    /** overloaded constructor. */
    Automobile(String sModelName, int iNumGallons, int MPG)
    {
        sModel = sModelName;
        iTankCapacity = iNumGallons;
        iMilesPerGallon = MPG;
        gasStorage = new FuelTank(this, iTankCapacity);
        System.out.println("Auto, model: " + sModel + ", capacity: " + iTankCapacity +
                           ", MPG: " + iMilesPerGallon);
    }

    /** method to move (drive) the auto. */
    void move(int iMiles)
    {
        for (int i=0; i < iMiles; i++)
        {
            if (gasStorage.gasLeft())
                iOdometer++;
            else
            {
                System.out.println("Ran out of gas.");
                System.out.println("Odometer: " + iOdometer);
                break;
            }

            if (iOdometer % 50 == 0)
            {
                System.out.println("Odometer: " + iOdometer);
                System.out.println("Gallons : " + gasStorage.getCurrentGallons());
            }
        }
    }
}

/**
 * Class to represent a fuel tank.
 */
class FuelTank
{
    /** capacity of fuel tank in gallons. */
    int iCapacity;
    /** current # of gallons in the tank. */
    float fCurrentGallons;
    /** Last odometer reading of "parent" automobile. */
    int iLastOdometerReading;
    /** The reference to the parent automobile. */
    Automobile parent;

    /** constructor. */
    FuelTank(Automobile theParent, int iInitialCapacity)
    {
        parent = theParent;
        iCapacity = iInitialCapacity;
        fCurrentGallons = iInitialCapacity;
        iLastOdometerReading = parent.iOdometer;
    }

    /** accessor. */
    float getCurrentGallons()
    { return fCurrentGallons; }

    /** calculates the current fuel in the tank. */
    float currentCapacity()
    {
        int iDistance = parent.iOdometer - iLastOdometerReading;

        if (iDistance > 0)
        {
            float fGallonsWasted = (float)iDistance / parent.iMilesPerGallon;
            if (fGallonsWasted > 0 && fCurrentGallons > 0)
            {
                fCurrentGallons -= fGallonsWasted;
            }
        }

        iLastOdometerReading = parent.iOdometer;
        return fCurrentGallons;
    }

    /** method to determine if there is any gas left? */
    boolean gasLeft()
    {
        return (currentCapacity() > 0) ? true : false;
    }
}

/**
 * Wrapper class to drive the demo.
 */
class ThisDemo
{
    /** main() method to invoke from a JVM. */
    public static void main(String args[])
    {
        Automobile newCar = new Automobile("Chevy");
        newCar.move(300);
    }
}