/** ListHead.java. */
package List;
import java.util.Enumeration;
import java.util.NoSuchElementException;

/** Class to implement a linked list manager. */
public class ListHead implements Enumeration, Traverse
{
    /** flag to denote type of nodes this list contains. */
	boolean doubly;
	/** reference to head of linked list. */
	ListNode head;
	/** reference to last node in linked list. */
	ListNode tail;
	/** number of nodes in list. */
	int count;

	/** Used to implement Enumeration. */
	int current;
	/** reference to current node. */
	ListNode curNode;

	/** Used to implement Traverse. */
	boolean forward;

    /** constructor. */
	public ListHead(boolean isDoubly)
	{
	    /* "typing" the list in this fashion will
	       insure a homogenous list. */
		doubly = isDoubly;
	}

    /** Add a data element to this linked list. */
	public synchronized void addElement(Object data)
	{
		ListNode inNode;

		if (doubly)
				inNode = new DoubleLink(data);
		else
				inNode = new SingleLink(data);

		if (head == null)
		{
				head = tail = inNode;
		}
		else
		{
				ListNode tmp = tail;
				tail = inNode;
				inNode.Link(tmp);
		}
		count++;
	}

    /** remove first element of the list. */
	public synchronized Object removeFirst()
	{
		if (head != null)
		{
				ListNode tmp = head;
				if (tail == head)
						tail = head.next();
				head = head.next();
				count--;
				return tmp.data;
		}
		return null;
	}

	/** get size of list. */
	public int size() { return count; }

	/** The Enumeration Interface */
	public boolean hasMoreElements()
	{
		if (count > 0 && current < count)
				return true;
		else
				return false;
	}

    /** implements the enumeration interface. */
	public Object nextElement()
	{
		forward = true;
		if (count > 0 && current == 0)
		{
				curNode = head;
		}
		else if (current > 0 && current < count)
		{
				if (forward)
						curNode = curNode.next();
		}
		else
				throw new NoSuchElementException();

		if (forward)
				current++;
		return curNode.data;
	}

    /** print all nodes in the list. */
	public void printAll()
	{
		ListNode traverse = head;

		for (int i=0; i < count; i++)
		{
			System.out.println("Node #" + i + " : "
							   + traverse.data);
			traverse = traverse.next();
		}
	}

    /** demonstration of package friendliness. */
	public void showFriendly()
	{
		SingleLink node1 = new SingleLink(new Integer(100));
		SingleLink node2 = new SingleLink(new Integer(200));

		node1.next = node2;
		System.out.println("Inside the package, ");
		System.out.println("  a class can access another classes ");
		System.out.println("  variables, unless they are private.");
		System.out.println("  node1.next.data is : " + node1.next.data);
	}

	// the Traverse interface...
	/** am I at the beginning? returns false for an empty list.*/
	public boolean atStart()
	{
		if (count > 0 && current <= 0)
		{
				forward = true; // only possible direction
				return true;
		}
		else
				return false;
	}

	/** am I at the end?. */
	public boolean atEnd()
	{
		if (count > 0 && current == count)
		{
				forward = false; // only possible direction
				return true;
		}
		else
				return false;
	}

	/** access the next list element. */
	public Object next()
	{
		forward = true;
		if (count > 0 && current == 0)
		{
			curNode = head;
		}
		else if (current > 0 && current < count)
		{
			if (forward)
				curNode = curNode.next();
		}
		else
			throw new NoSuchElementException();

		if (forward)  // handle direction change
			current++;
		return curNode.data;
	}

	/** access the previous list element. */
	public Object prev() throws IllegalListOpException
	{
		if (doubly)
		{
			if (count > 0 && current == count)
			{
				curNode = tail;
			}
			else if (count > 0 && current > 0)
			{
				if (forward == false)  // moving backward
				{
					try {
					curNode = curNode.prev();
					} catch (Exception e) { }
				}
			}
			else if (count == 0 || current == 0)
			{
				throw new IllegalListOpException("Prev() invalid for empty list.");
			}
			else
				throw new IllegalListOpException("Prev() invalid.");
		}
		else
		{
			// undefined for singly linked lists
			throw new IllegalListOpException("Prev() invalid for singly linked list.");
		}

		if (!forward)  // handle direction change
			current--;
		forward = false;
		return curNode.data;
	}

	/** jump to the head of the list. */
	public Object start()
	{
		if (count > 0)
		{
			current = 0;
			curNode = head;
			forward = true;
		}
		else
			throw new NoSuchElementException();
		return curNode.data;
	}

	/** jump to the last element of the list. */
	public Object end()
	{
		if (count > 0)
		{
			current = count - 1;
			curNode = tail;
			forward = false;
		}
		else
			throw new NoSuchElementException();
			return curNode.data;
	}
}

