/** 
 * dateTime.c
 * C function which is called by java, updates a java variable and then calls a java method.
 *
 * Al Saganich, Java for C/C++ programmers.
 */

#include <stdio.h>
#include <jni.h>
#include <time.h>
#include "dateTime.h"  // Header file produced by javap -s -p someclass

/*
* Out native method.
*/
JNIEXPORT void JNICALL Java_dateTime_nmFillDateTime (JNIEnv *pJNIEnv , jobject jObj)
{
	jclass jclsContainingClass;
	jmethodID methodID; 
	jfieldID fieldID;
	jstring jString;
	const char *pCStr;
	time_t ltime;


	/*
	* Get a handle to the containing class.
	*/
	jclsContainingClass = (*pJNIEnv)->GetObjectClass(pJNIEnv, jObj);

	/* java.lang.String dateTime;
	* Ljava/lang/String;   
	*/
	printf("C:Getting handle to data member dateTime\n");
	fieldID = (*pJNIEnv)->GetFieldID(pJNIEnv, jclsContainingClass, "dateTime", "Ljava/lang/String;");
	if (fieldID == 0)
	{
		printf("C:Error field  dateTime not found!\n");
		return;
	}

	
	// Access the java string
	jString = (*pJNIEnv)->GetObjectField(pJNIEnv, jObj, fieldID);
	// get a c-string pointer
	pCStr = (*pJNIEnv)->GetStringUTFChars(pJNIEnv, jString, 0);
	
	printf("C:Original value of dateTime = %s\n",pCStr);

	// Done with it
	(*pJNIEnv)->ReleaseStringUTFChars(pJNIEnv, jString, pCStr);

	// Now update the java field with the current time.
	time( &ltime );
	printf("C:setting dateTime = %s\n",ctime( &ltime ));
	// Allocate a java string to contain the new value
  	jString= (*pJNIEnv)->NewStringUTF(pJNIEnv, ctime( &ltime ));
	// set the value into our string
    (*pJNIEnv)->SetObjectField(pJNIEnv, jObj, fieldID, jString);
 

	/*
	*	 Get a handle to the method to call
	*    void javaPrintDateTime(java.lang.String);
	*	   (Ljava/lang/String;)V  
	*/
	methodID = (*pJNIEnv)->GetMethodID(pJNIEnv, jclsContainingClass, "javaPrintDateTime", "(Ljava/lang/String;)V");
	if (methodID == 0)
	{
		printf("C:Error method javaPrintDateTime not found!\n");
		return;
	}

	(*pJNIEnv)->CallVoidMethod(pJNIEnv, jObj, methodID, jString);
	printf("C:Called Java method returning to Java.\n");
}