// ClassCreator.java
package jwiley.chp6;

import java.lang.reflect.*;

/**
 * Class which creates objects of arbitrary types as long
 * as either a default constructor or a constructor which
 * takes a single string argument is available
 */
public class ClassCreator
{
    /** Private constructor for demonstration purposes
     */
    private ClassCreator(String strArg) {}

    /**
     * Creates and returns an object of an arbitrary type.
     * If the class doesn't have a default constructor or
     * a constructor which takes a single string argument
     * then an exception is thrown.
     */
    public static Object CreateClass(String strClassName) throws
      IllegalAccessException,
      InvocationTargetException,
      InstantiationException,
      ClassNotFoundException
    {
        // First we need to load the class
        Class c = Class.forName(strClassName);

        // Now, enumerate the constructors
        Constructor constructors[] = c.getDeclaredConstructors();

        // first look for a constructor which takes zero args
        for (int i= 0; i < constructors.length; i++)
        {
            if (constructors[i].getParameterTypes().length == 0)
            {
                return constructors[i].newInstance(null);
            }
        }

        // Ok, that didn't work, now look for a constructor which
        // takes a single string.  We'll pass in an empty string ""
        for (int i= 0; i < constructors.length; i++)
        {
            Class classString = Class.forName("java.lang.String");
            Class paramTypes[] = constructors[i].getParameterTypes();

            if (paramTypes.length == 1 &&
                classString.equals(paramTypes[0]))
            {
                Object args[] = new Object[1];
                args[0] = new String("");
                return constructors[i].newInstance(args);
            }
        }

        throw new InstantiationException(
           "Couldn't find appropriate constructor");
    }

    /**
     * Main routine so the class can be tested from the command line.
     * A class name is expected to be passed in on the command line
     */
    public static void main(String argv[]) throws Exception
    {
        if (argv.length == 0)
        {
            System.out.println("USAGE: java ClassCreator <class name>");
            return;
        }

        try
        {
            Object obj = CreateClass(argv[0]);
            System.out.println("Class created successfully!");
        } catch (Exception ex)
          {
            ex.printStackTrace(System.out);
          }
    }
}
