// ClassInvoker.java
package jwiley.chp6;

import java.lang.reflect.*;

/**
 * Uses reflection to examine the structure of a class for specific features
 * if the class has all of the expected features, then an
 * object is created, a field is examined, and a method is
 * invoked.  For simpler testing, this class implements the
 * exact features it requires in order to invoke the method.
 */
public class ClassInvoker
{
    /**
     * String data member for ClassInvoker.  This member only
     * exists so that ClassInvoker will support the features required
     * of classes it will work with.
     */
    public String strMsg = "This is a very complex way to call a method!";

    /**
     * A method which takes a single string as an argument.  This
     * method only exists so that ClassInvoker will support the
     * features required of classes it will work with.
     */
    public void TestMethod(String strArg)
    {
        System.out.println(strArg);
    }

    /**
     * Uses reflection to examine a class for the following two properties:
     * 1) An intance variable which is a String reference
     * 2) A method which takes a single string as an argument
     * If the class meets those criteria, then an object is created,
     * and the method is called, using the string field as an argument
     * to the method.
     */
    public static void InvokeClass(String strClassName) throws Exception
    {
        Object obj = ClassCreator.CreateClass(strClassName);
        Class c = Class.forName(strClassName);
        Class classString = Class.forName("java.lang.String");

        Field theField = null;
        Method theMethod = null;

        // find the first instance variable on the class which is a string
        Field fields[] = c.getFields();
        for (int i = 0; i < fields.length; i++)
        {
            if (!Modifier.isStatic(fields[i].getModifiers()) &&
                classString.equals(fields[i].getType()))
            {
                theField = fields[i];
                break;
            }
        }

        if (theField == null)
        {
            throw new InstantiationException(
                        "Could not find a valid field");
        }

        // find the first method which takes a single string argument
        Method methods[] = c.getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            if (!Modifier.isStatic(methods[i].getModifiers()))
            {
                Class params[] = methods[i].getParameterTypes();
                if (params.length == 1 && classString.equals(params[0]))
                {
                    theMethod = methods[i];
                    break;
                }
            }
        }

        if (theMethod == null)
        {
            throw new InstantiationException(
                        "Could not find a valid field");
        }

        Object args[] = new Object[1];
        args[0] = theField.get(obj);

        theMethod.invoke(obj, args);
    }

    /**
     * Allows users to invoke the ClassInspector from the command
     * line.  This routine expects the name of the class to inspect
     * to be entered as a command line argument
     */
    public static void main(String argv[]) throws Exception
    {
        if (argv.length == 0)
        {
            System.out.println("USAGE: java ClassInvoker <class name>");
            return;
        }

        InvokeClass(argv[0]);
    }
}
