// ControllerFrame.java
package jwiley.chp7;

import jwiley.chp10.Grid;
import java.awt.*;
import java.awt.event.*;

/**
 * This class is used as the container for all of the components
 * It relays button presses to ModelUpdater so the PositionModel
 * can be updated.
 *
 * @author  Eric Monk
 * @version 1.0
 */

public class ControllerFrame extends Frame implements MouseListener
{
    Button  cmdWest = new Button("W");
    Button  cmdNW = new Button("NW");
    Button  cmdNorth = new Button("N");
    Button  cmdNE = new Button("NE");
    Button  cmdEast = new Button("E");
    Button  cmdSE = new Button("SE");
    Button  cmdSouth = new Button("S");
    Button  cmdSW = new Button("SW");

    PositionModel   pm;
    Thread          t;
    ModelUpdater    modelUpdater;

    public ControllerFrame (String sTitle)
    {
        super(sTitle);

        modelUpdater = new ModelUpdater(this);
        t = new Thread(modelUpdater);
        t.start();

        setLayout(new Grid(3,1));
        setBackground(Color.lightGray);

        // Control Panel
        Panel pnlControl = new Panel();
        pnlControl.setLayout(new Grid(3,5,2,2));

        pnlControl.add("1,3", cmdWest);
        pnlControl.add("1,2", cmdNW);
        pnlControl.add("2,2", cmdNorth);
        pnlControl.add("3,2", cmdNE);
        pnlControl.add("3,3", cmdEast);
        pnlControl.add("3,4", cmdSE);
        pnlControl.add("2,4", cmdSouth);
        pnlControl.add("1,4", cmdSW);

        cmdWest.addMouseListener(this);
        cmdNW.addMouseListener(this);
        cmdNorth.addMouseListener(this);
        cmdNE.addMouseListener(this);
        cmdEast.addMouseListener(this);
        cmdSE.addMouseListener(this);
        cmdSouth.addMouseListener(this);
        cmdSW.addMouseListener(this);

        // View Panel
        Panel pnlView = new Panel();
        pnlView.setLayout(new Grid(3,5,2,2));

        pm = new PositionModel(new Rectangle(400,400), new Point(200,200));

        AerialView aerialView = new AerialView(pm);
        SideView topView = new SideView(SideView.TOP_BOTTOM_VIEW, pm);
        SideView leftView = new SideView(SideView.LEFT_RIGHT_VIEW, pm);

        pm.addObserver(aerialView);
        pm.addObserver(topView);
        pm.addObserver(leftView);

        pnlView.add("1,1", new Label("Aerial View"));
        pnlView.add("2,1,2,3", aerialView.getCanvas());

        pnlView.add("1,4", new Label("Top View"));
        pnlView.add("2,4,2,1", topView.getCanvas());

        pnlView.add("1,5", new Label("Left View"));
        pnlView.add("2,5,2,1", leftView.getCanvas());

        // Layout Panels
        add("1,1,2,1", pnlView);
        add("3,1", pnlControl);

        setSize(400,250);
        show();
    }

    public void mouseClicked(MouseEvent evt)    {   }
    public void mouseEntered(MouseEvent evt)    {   }
    public void mouseExited(MouseEvent evt)     {   }

    public void mousePressed(MouseEvent evt)
    {
        modelUpdater.setTarget(evt.getSource());
        modelUpdater.setRun(true);
    }

    public void mouseReleased(MouseEvent evt)
    {
        modelUpdater.setRun(false);
    }

    public static void main (String args[])
    {
        new ControllerFrame("Observable/Observer example");
    }
}

