
package jwiley.chp7;

import java.awt.*;

public class SimpleSlider extends Canvas
{
    int     iOldPosition;
    int     iNewPosition;
    int     iMarkerWidth;
    Color   colorBack = Color.white;
    Color   colorMarker = Color.black;

    Graphics    offScreenGraphics;
    Image       offScreenImage;

    public SimpleSlider (int iInitPosition, int iMarkerWidth)
    {
        if (iMarkerWidth < 1)
            iMarkerWidth = 1;

        if (iInitPosition < 0)
            iInitPosition = 0;
        else if (iInitPosition > 100)
            iInitPosition = 100;

        this.iMarkerWidth = iMarkerWidth;
        setBackground(colorBack);

        iOldPosition = iInitPosition;
    }

    public void paint (Graphics g)
    {
        if (offScreenImage != null)
        {
            g.drawImage(offScreenImage, 0,0,this);
        }
        else
        {
            setMarker(iOldPosition);
            draw(iNewPosition, g);
            iOldPosition = iNewPosition;
        }

    }

    public void update (Graphics g)
    {
        if (g != null && offScreenGraphics != null)
        {
            if (iOldPosition != iNewPosition)
            {
                offScreenGraphics.setXORMode(colorBack);
                draw(iOldPosition, offScreenGraphics);
                offScreenGraphics.setPaintMode();
                offScreenGraphics.setColor(colorMarker);
                draw(iNewPosition, offScreenGraphics);
                iOldPosition = iNewPosition;
            }
            else
                draw(iOldPosition, offScreenGraphics);

            paint(g);
        }
        else
            createOffScreenGraphics();
    }

    public void createOffScreenGraphics()
    {
        Dimension d = getSize();
        offScreenImage = createImage(d.width, d.height);
        offScreenGraphics = offScreenImage.getGraphics();
        draw(iOldPosition, offScreenGraphics);
    }

    public void draw (int iPosition, Graphics g)
    {
        int iXPosition = iPosition - (int)(iMarkerWidth / 2);
        g.fillRect(iXPosition, 0, iMarkerWidth, this.getSize().height);
    }

    public void setSize(Dimension d)
    {
        setSize(d.width, d.height);
    }

    public void setSize(int x, int y)
    {
        super.setSize(x, y);
        createOffScreenGraphics();
    }

    public void setMarker (int iPercent)
    {
        if (iPercent < 0)
            iPercent = 0;
        else if (iPercent > 100)
            iPercent = 100;

        iNewPosition = (int)(this.getSize().width * iPercent * .01);
        repaint();
    }

    public static void main (String args[])
    {
        Frame f = new Frame();
        SimpleSlider ss = new SimpleSlider(50, 6);
        f.setLayout(new BorderLayout());
        f.add("Center", ss);
        f.setSize(200,50);
        f.show();

        try {   Thread.sleep(2000);  } catch (Exception e)   {   }

        for (int iCount = 0; iCount < 100; iCount++)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            ss.setMarker(iCount);
        }

        for (int iCount = 100; iCount > 0; iCount--)
        {
            try {   Thread.sleep(30);  } catch (Exception e)   {   }
            ss.setMarker(iCount);
        }
    }
}
