// CommentFilter.java
package jwiley.chp8;

import java.io.FilterInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.PushbackInputStream;
import java.io.IOException;
import java.util.Stack;
import java.lang.Character;

/** Class to demonstrate FilterInputStream. */
public class CommentFilter extends FilterInputStream
{
    /** flag to determine whether to output comments to a file. */
    private boolean OutputComments=false;
    /** Stream to store comments. */
    private FileOutputStream fos;
    /** Stream to format outgoing text. */
    private PrintWriter pw;
    /** characters in the stream. */
    private static char currentChar=0, lastChar=0, nextChar=0;
    /** flag to determine if in a single line comment. */
    private static boolean inLineComment=false;
    /** flag to determine if in a multiLineComment. */
    private static boolean inMultiLineComment=false;

    /** one arg constructor. */
    CommentFilter(InputStream src)
    {
            super(src);
            PushbackInputStream pis = new PushbackInputStream(src);
            in = pis;
    }

    /** open a potential output stream for comments
       if output flag is set */
    CommentFilter(InputStream src, String outFileName)
    {
            super(src);
            PushbackInputStream pis = new PushbackInputStream(src);
            in = pis;
            OutputComments = true;

            try {
                    fos = new FileOutputStream(outFileName);
                    pw = new PrintWriter(fos);
            } catch (IOException e)
              {
                    System.out.println("IO Error on " + outFileName);
                    System.exit(1);
              }
              catch (Exception e)
              {
                    System.out.println("Unable to filter input");
                    System.exit(1);
              }
    }

    /** override the read() method in FilterInputStream. */
    public int read()
    {
            int input=0;
            boolean peekAhead = false;

            try {
                input = super.read();
                if (input == -1)
                    return -1;
                currentChar = (char) input;
            } catch (IOException ioe) { return -1; }

            if (currentChar == '/' && !inLineComment &&
                !inMultiLineComment)
            {
                // peek ahead
                try {
                    input = super.read();
                    if (input == -1)
                            return -1;
                   nextChar = (char) input;
                   peekAhead = true;
                } catch (IOException ioe) { return -1; }

                if (nextChar != '/' && nextChar != '*')
                {
                    // push it back on the stream for next read
                    try {
                      ((PushbackInputStream)in).unread(nextChar);
                    } catch (IOException ioe) { return -1; }
                }
            }

            if (currentChar == '/' && nextChar == '*')
            {
                    if (!inLineComment)
                       inMultiLineComment = true;
            }
            else if (currentChar == '/' && nextChar == '/')
            {
                    if (!inMultiLineComment)
                       inLineComment = true;
            }

            if (inLineComment || inMultiLineComment)
            {
               while (inLineComment || inMultiLineComment)
               {
                    if (OutputComments)
                    {
                            pw.print(currentChar);
                            if (peekAhead)
                            {
                                    pw.print(nextChar);
                                    peekAhead = false;
                            }
                            pw.flush();
                    }

                    // check if comment is over
                    if (inMultiLineComment && lastChar == '*' &&
                        currentChar == '/')
                    {
                            inMultiLineComment = false;
                            if (OutputComments)
                            {
                                    pw.print('\n');
                                    pw.flush();
                            }
                    }

                    if (inLineComment &&
                        (currentChar == 10 || currentChar == 13))
                    {
                            if (OutputComments)
                            {
                                    pw.print('\n');
                                    pw.flush();
                            }

                            inLineComment = false;
                            lastChar = currentChar;
                            return currentChar;
                    }

                    lastChar = currentChar;

                    // get next character
                    try {
                        input = super.read();
                        if (input == -1)
                            return -1;
                      currentChar = (char) input;
                    } catch (IOException ioe) { return -1; }
               }
            }
            else
                    return currentChar;

            return currentChar;
    }

    /** close the file prior to object destruction. */
    public void finalize()
    {
        try
        {
            fos.close();
        } catch (Exception e) { }
    }

    /** main() method to invoke from JVM. */
    public static void main(String args[])
    {
        // Get command line argument
        if (args.length < 1)
        {
                System.out.println("USAGE: java CommentFilter fileToRead [-output commentFile]");
                System.out.println("  -output flag is optional.");
                System.exit(1);
        }

        String fileName = null;
        String commentFileName = null;
        boolean captureComments = false;
        for (int i=0; i < args.length; i++)
        {
                if (args[i].equals("-output"))
                {
                        if (args.length > i + 1)
                        {
                                commentFileName = new String(args[i+1]);
                                i++;
                                captureComments = true;
                        }
                        else
                        {
                                System.out.println("No output file specified.");
                                System.exit(1);
                        }
                }
                else
                        fileName = new String(args[i]);
        }

        System.out.println("fileName is " + fileName);

        FileInputStream fis=null;
        // open the input file
        try
        {
            fis = new FileInputStream(fileName);
        } catch (FileNotFoundException fnf)
          {
                System.out.println("Unable to open " + fileName);
                System.exit(1);
          }

        if (captureComments)
                System.out.println("Comment File is " + commentFileName);

        CommentFilter cf=null;
        if (captureComments)
                cf = new CommentFilter(fis, commentFileName);
        else
                cf = new CommentFilter(fis);

        int in=0;
        while ( (in = cf.read()) != -1)
        {
                System.out.print((char)in);
                System.out.flush();
        }

        try
        {
          fis.close();
        } catch (Exception e) { }
    }
}
